@@ -134,8 +134,11 @@ class Core(component.Component):
         # store the one in the config so we can restore it on shutdown
         self.__old_interface = None
         if listen_interface:
-            self.__old_interface = self.config["listen_interface"]
-            self.config["listen_interface"] = listen_interface
+            if deluge.common.is_ip(listen_interface):
+                self.__old_interface = self.config["listen_interface"]
+                self.config["listen_interface"] = listen_interface
+            else:
+                log.error("Invalid listen interface (must be IP Address): %s", listen_interface)
 
     def start(self):
         """Starts the core"""
@@ -277,7 +280,7 @@ class Core(component.Component):
                 result.addCallbacks(on_download_success, on_download_fail)
             else:
                 # Log the error and pass the failure onto the client
-                log.error("Error occured downloading torrent from %s", url)
+                log.error("Error occurred downloading torrent from %s", url)
                 log.error("Reason: %s", failure.getErrorMessage())
                 result = failure
             return result
