@@ -39,13 +39,20 @@ gobject.set_prgname("deluge")
 
 # Install the twisted reactor
 from twisted.internet import gtk2reactor
-reactor = gtk2reactor.install()
+from twisted.internet.error import ReactorAlreadyInstalledError
+
+try:
+    reactor = gtk2reactor.install()
+except ReactorAlreadyInstalledError, e:
+    # Running unit tests so trial already installed a rector
+    pass
 
 import gettext
 import locale
 import pkg_resources
 import gtk, gtk.glade
 import sys
+import warnings
 
 try:
     from setproctitle import setproctitle, getproctitle
@@ -177,7 +184,15 @@ class GtkUI(object):
         try:
             import gnome.ui
             import gnome
-            self.gnome_prog = gnome.init("Deluge", deluge.common.get_version())
+
+            #Suppress: Warning: Attempt to add property GnomeProgram::*** after class was initialised
+            original_filters = warnings.filters[:]
+            warnings.simplefilter("ignore")
+            try:
+                self.gnome_prog = gnome.init("Deluge", deluge.common.get_version())
+            finally:
+                warnings.filters = original_filters
+
             self.gnome_client = gnome.ui.master_client()
             def on_die(*args):
                 reactor.stop()
@@ -198,8 +213,8 @@ class GtkUI(object):
             SetConsoleCtrlHandler(win_handler)
 
         if deluge.common.osx_check() and gtk.gdk.WINDOWING == "quartz":
-            import gtk_osxapplication
-            self.osxapp = gtk_osxapplication.OSXApplication()
+            import gtkosx_application
+            self.osxapp = gtkosx_application.gtkosx_application_get()
             def on_die(*args):
                 reactor.stop()
             self.osxapp.connect("NSApplicationWillTerminate", on_die)
