@@ -584,7 +584,7 @@ class ServerContextFactory:
 
     def getContext(self):
         """Creates an SSL context."""
-        ctx = SSL.Context(SSL.SSLv3_METHOD)
+        ctx = SSL.Context(SSL.TLSv1_METHOD)
         deluge_web = component.get("DelugeWeb")
         log.debug("Enabling SSL using:")
         log.debug("Pkey: %s", deluge_web.pkey)
